from django import forms

from benevalibre.benevalo.models import Benevalo
from benevalibre.utils.fields import SplitDurationField
from benevalibre.utils.forms import CustomTapeformMixin


class AssociatedEngagedBenevaloCreateForm(
    CustomTapeformMixin, forms.ModelForm
):
    layout_template = 'benevalo/forms/layouts/benevalo.html'

    def __init__(self, *args, **kwargs):
        self.association = kwargs.pop('association')

        super().__init__(*args, **kwargs)
        self.fields['category'].queryset = self.association.category_set.all()

        if not self.association.project_set.exists():
            del self.fields['project']
        else:
            self.fields[
                'project'
            ].queryset = self.association.project_set.all()

        if not self.association.level_set.exists():
            del self.fields['level']
        else:
            self.fields['level'].queryset = self.association.level_set.all()

    class Meta:
        model = Benevalo
        fields = [
            'title',
            'duration',
            'distance',
            'execution_date',
            'execution_end_date',
            'category',
            'project',
            'level',
            'description',
        ]
        widgets = {'description': forms.Textarea(attrs={'rows': '4'})}
        field_classes = {'duration': SplitDurationField}


class AssociatedEngagedBenevaloUpdateForm(AssociatedEngagedBenevaloCreateForm):
    pass


class AssociatedBenevaloCreateForm(AssociatedEngagedBenevaloCreateForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['user'].queryset = self.association.get_benevoles()

    class Meta(AssociatedEngagedBenevaloCreateForm.Meta):
        fields = ['user'] + AssociatedEngagedBenevaloCreateForm.Meta.fields


class AssociatedBenevaloUpdateForm(AssociatedBenevaloCreateForm):
    pass


class AssociationEngageForm(forms.Form):
    pass
