from django.urls import reverse

import django_tables2 as tables


class ActionsColumn(tables.TemplateColumn):
    default_extra = {
        'attrs': {'td': {'class': 'actions-column'}},
        'exclude_from_export': True,
        'orderable': False,
        'verbose_name': '',
    }

    def __init__(self, update_url=None, delete_url=None, **extra):
        if not update_url and not delete_url:
            raise ValueError("At least one action's URL must be defined")
        self.update_url = update_url
        self.delete_url = delete_url
        # set default values for extra arguments
        extra = dict(self.default_extra, **extra)
        if 'template_code' not in extra and 'template_name' not in extra:
            extra['template_name'] = 'tables/actions_column.html'
        super().__init__(**extra)

    def render(self, record, *args, **kwargs):
        for name in ('update_url', 'delete_url'):
            self.extra_context[name] = self.resolve_url_for_record(
                getattr(self, name), record
            )
        return super().render(record, *args, **kwargs)

    def resolve_url_for_record(self, value, record):
        if value is None:
            return None
        elif callable(value):
            value = value(record)
        elif isinstance(value, str):
            if not value.startswith(('/', './', '../')):
                value = reverse(value, args=(record.pk,))
        else:
            raise ValueError("Unexpected value '{}'".format(repr(value)))
        return value
